//=========================================================================
//Copyright LizardOfOz.
//=========================================================================

::listenerTable <- {};

//The higher the value the LATER it will execute
::AddListener <- function(eventName, order, listenerFunc, scope = null)
{
    if (scope == null)
        scope = this;
    local listenerEntry = [order, listenerFunc, scope];
    if (eventName in listenerTable)
    {
        local listenerQueue = listenerTable[eventName];
        local size = listenerQueue.len();
        local i = 0;
        for (; i < size; i++)
            if (listenerQueue[i][0] > order)
                break;
        listenerQueue.insert(i, listenerEntry);
    }
    else
        listenerTable[eventName] <- [listenerEntry];
    return listenerEntry;
}

//Removing listeners is not really intended,
//  but in case you need it, you can store the return of `AddListener` in a variable,
//  and then pass that variable to `RemoveListener`.
::RemoveListener <- function(listenerEntry)
{
    foreach (listenerQueue in listenerTable)
    {
        local index = listenerQueue.find(listenerEntry);
        if (index)
        {
            listenerQueue.remove(index);
            return;
        }
    }
}
//todo make listeners from non-existent entities delete themselves.

::FireListeners <- function(eventName, params)
{
    if (!(eventName in listenerTable))
        return;

    local player = null;
    if ("userid" in params)
    {
        player = GetPlayerFromUserID(params.userid);
        if (!IsValidClient(player))
            return;
    }
    local listenerQueue = listenerTable[eventName];
    foreach (listenerEntry in listenerQueue)
    {
        if (listenerEntry[2] == null)
        {
            queue.remove(listenerEntry);
            continue;
        }
        try { listenerEntry[1].call(listenerEntry[2], player, params); }
        catch (e) { throw e; }
    }
}

::OnGameEventInner <- function(eventName, order, listenerFunc, scope, prefix, eventType, regFunc)
{
    local eventTable = root_table[eventType];
    if (!(eventName in eventTable))
    {
        root_table[prefix + eventName] <- function(params) { FireListeners(eventName, params); }
        eventTable[eventName] <- [];
        regFunc(eventName);
    }
    if (eventTable[eventName].find(scope) == null)
        eventTable[eventName].append(scope.weakref());
    AddListener(eventName, order, listenerFunc, scope);
}

::OnCustomEvent <- AddListener;
::FireCustomEvent <- FireListeners;

::OnGameEvent <- function(eventName, order, listenerFunc, scope = this)
{
    if (endswith(eventName, "_post"))
        OnGameEventPost(eventName.slice(0, eventName.len() - 5), order, listenerFunc, scope);
    else if (eventName == "OnTakeDamage" || eventName == "take_damage" || eventName == "damage_hook")
        OnGameHook("OnTakeDamage", order, listenerFunc, scope = this);
    else
        OnGameEventInner(eventName, order, listenerFunc, scope, "OnGameEvent_", "GameEventCallbacks", RegisterScriptGameEventListener);
}

::OnGameHook <- function(hookName, order, listenerFunc, scope = this)
{
    OnGameEventInner(hookName, order, listenerFunc, scope, "OnScriptHook_", "ScriptHookCallbacks", RegisterScriptHookListener);
}

::OnGameEventPost <- function(eventName, order, listenerFunc, scope = this)
{
    OnGameEvent(eventName, order, function(player, params) {
        RunWithDelay(scope, -1, listenerFunc, player, params);
    });
}