//=========================================================================
//Copyright LizardOfOz.
//=========================================================================

PrecacheSound("vo/announcer_failure.mp3");
PrecacheSound("vo/announcer_success.mp3");
::death_penalty <- 30;

OnGameEvent("player_death", 0, function(player, params)
{
    local isVIP = IsVIP(player);
    player.SetCustomModelWithClassAnimations("");
    ClearVIP(player);

    if (isVIP)
        FireCustomEvent("vip_death", params);
});

OnCustomEvent("vip_death", 0, function(player, params)
{
    local attacker = GetPlayerFromParams(params, "attacker");
    if (attacker && attacker != player && GetPropInt(tf_gamerules, "m_iRoundState") == 4)
    {
        local timeRemaining = GetPropFloat(timer, "m_flTimerEndTime") - Time() - 5;
        EntFireByHandle(team_round_timer, "AddTime", "-" + min(timeRemaining, death_penalty), 0, null, null);
        foreach(otherPlayer in GetAllClients())
        {
            ClientPrint(otherPlayer, Constants.EHudNotify.HUD_PRINTTALK, "\x07FFCC22VIP died!");
            EmitSoundEx({
                sound_name = player.GetTeam() == otherPlayer.GetTeam() ? "vo/announcer_failure.mp3" : "vo/announcer_success.mp3",
                filter_type = Constants.EScriptRecipientFilter.RECIPIENT_FILTER_SINGLE_PLAYER
                flags =  1,
                channel = 7
                entity = otherPlayer,
                speaker_entity = otherPlayer
            });
        }
    }
});

OnGameEvent("player_spawn", 0, function(player, params)
{
    player.SetCustomModelWithClassAnimations("");
    ClearVIP(player);
});

function ClearVIP(player)
{
    player.KeyValueFromString("targetname", "");
    player.SetCustomModelWithClassAnimations("");
    for(local ent = null; ent = Entities.FindByName(ent, "civ_model*");)
        if (ent && ent.IsValid() && ent.GetOwner() == player)
            ent.Kill();
    if (IsVIP(player))
    {
        vipPlayer = null;
        DoEntFire("civ_model*", "Kill", "", 0, null, null);
        DoEntFire("bomb_trigger", "FireUser1", "", 0, null, null);
    }
}

OnGameEvent("post_inventory_application", 0, function(player, params)
{
    if (IsVIP(player))
        ConvertToVIP0(player);
});

function ConvertToVIP(player)
{
    player.SetPlayerClass(TF_CLASS_DEMOMAN);
    vipPlayer = player;

    local vipDispenser = SpawnEntityFromTable("prop_dynamic",
    {
        origin = player.GetCenter(),
        angles = "-20 0 0",
        model = API_GetString("cane_pickup_model"),
        vscripts = "vip/entities/vip_dispenser.nut",
        thinkfunction = "DispenserThink",
        skin = player.GetTeam() == TF_TEAM_RED ? 0 : 1,
        playbackrate = 1.0,
        TeamNum = player.GetTeam()
    });
    EntFireByHandle(vipDispenser, "SetParent", "!activator", 0, player, player);

    ConvertToVIP0(player);
}

function ConvertToVIP0(player)
{
    SetPropInt(player, "m_Shared.m_bShieldEquipped", 0);
    vipBoostTokens <- vipBoostTokensStart;
    player.KeyValueFromString("targetname", "vip");

    player.SetCustomModelWithClassAnimations(API_GetString("vip_model"));

    local itemsToKill = [];
    for (local item = player.FirstMoveChild(); item != null; item = item.NextMovePeer())
        if (startswith(item.GetClassname(), "tf_we"))
            itemsToKill.push(item);
    foreach (item in itemsToKill)
        item.Kill();

    player.AddCustomAttribute("voice pitch scale", 0, -1);
    player.AddCustomAttribute("dmg taken from fire reduced", 0.8, -1);
    player.AddCustomAttribute("dmg taken from crit reduced", 0.8, -1);

    player.SetHealth(200);
    player.SetMaxHealth(200);
    player.RemoveCustomAttribute("max health additive bonus");
    player.AddCustomAttribute("max health additive bonus", 25, -1);

    for(local ent = null; ent = Entities.FindByName(ent, "civ_model*");)
        ent.Kill();

    local weapon = GivePlayerWeapon(player, "tf_weapon_bat", 190);
    weapon.SetModelSimple(API_GetString("vip_arms_model"));
    weapon.SetCustomViewModelModelIndex(GetModelIndex(API_GetString("vip_arms_model")));
    SetPropInt(weapon, "m_iViewModelIndex", GetModelIndex(API_GetString("vip_arms_model")));
    weapon.DisableDraw();

    local stockViewmodelEnt = GetPropEntity(player, "m_hViewModel");
    stockViewmodelEnt.DisableDraw();
    stockViewmodelEnt.SetModelSimple(API_GetString("vip_arms_model"));

    local customViewWeaponModel = TrySpawnViewModel(player);
    customViewWeaponModel.SetModelSimple(API_GetString("vip_arms_model"));

    local customViewWeaponModel = TrySpawnViewModel(player);
    customViewWeaponModel.SetModelSimple(API_GetString("umbrella_model"));

    local customViewWeaponModel = TrySpawnWorldModel(player, API_GetString("umbrella_model"));
    local hat = TrySpawnWorldModel(player, API_GetString("vip_hat_model"));
    hat.KeyValueFromString("targetname", "civ_model_hat");

    headshotProtection <- true;
}

::GivePlayerWeapon <- function(player, className, itemID)
{
    local weapon = Entities.CreateByClassname(className);
    NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemID);
    NetProps.SetPropBool(weapon, "m_AttributeManager.m_Item.m_bInitialized", true);
    NetProps.SetPropBool(weapon, "m_bValidatedAttachedEntity", true);
    weapon.SetTeam(player.GetTeam());
    Entities.DispatchSpawn(weapon);

    player.Weapon_Equip(weapon);

    for (local i = 0; i < 7; i++)
    {
        local heldWeapon = NetProps.GetPropEntityArray(player, "m_hMyWeapons", i);
        if (heldWeapon == null)
            continue;
        if (heldWeapon.GetSlot() != weapon.GetSlot())
            continue;
        heldWeapon.Destroy();
        NetProps.SetPropEntityArray(player, "m_hMyWeapons", weapon, i);
        break;
    }

    player.Weapon_Switch(weapon);

    return weapon;
}

::TrySpawnViewModel <- function(player)
{
    local playerIndex = player.entindex();

    local viewModel = SpawnEntityFromTable("tf_wearable_vm", {
        targetname = "civ_model"
    });
    SetPropBool(viewModel, "m_bForcePurgeFixedupStrings", true);
    player.EquipWearableViewModel(viewModel);

	return viewModel;
}

::TrySpawnWorldModel <- function(player, model)
{
    local playerIndex = player.entindex();
    local worldModel = SpawnEntityFromTable("tf_wearable", {
        targetname = "civ_model"
    });
    SetPropBool(worldModel, "m_bForcePurgeFixedupStrings", true);
    SetPropBool(worldModel, "m_bValidatedAttachedEntity", true);
    SetPropBool(worldModel, "m_AttributeManager.m_Item.m_bInitialized", true);
	worldModel.SetModelSimple(model);
    EntFireByHandle(worldModel, "SetParent", "!activator", 0, player, player);
	SetPropEntity(worldModel, "m_hOwnerEntity", player);
    worldModel.SetOwner(player);
    player.EquipWearableViewModel(worldModel);

	return worldModel;
}